#include "FaspManagerSample.h"
#include "FaspManagerException.h"

/**
 * Transfer parameters 
 */
#define TARGET_RATE          10000    /* Target (max) transfer rate in Kbps */
#define MIN_RATE             5000     /* Minimum transfer rate in Kbps - adaptive rate only */
#define CREATE_DIRS          false    /* Create the target directory if it does not exist   */

int main( )
{
	Aspera::FaspManager::JobOrder *job_defn;
    Aspera::FaspManager::Location *src;
    Aspera::FaspManager::Location *dst;
    Aspera::FaspManager::XferParams *xfer_params;
    Aspera::FaspManager::Manager *manager;
	
	printf("\nSimple FaspManager Download Sample:\n\n");

    /*
    * Configure the fasp Manager environment. The fasp manager environment 
    * can be configured using the set methods provided by Environment class. 
    * Depending on your deployment scenario you need to set one or more of
    * these. Refer to the documentation bundled with the SDK for more details.
    */
	try {
		// These should get detected but can be overidden
	    // Aspera::FaspManager::Environment::SetFasp2AscpPath("C:\\Program Files (x86)\\Aspera\\Enterprise Server\\bin\\ascp.exe");
	    // Aspera::FaspManager::Environment::SetPortFilePath("C:\\Program Files\\Aspera\\Enterprise Server\\var\\run\\aspera\\fmsample.optport");

	    /* Setup the Local and Remote endpoints for a download */
		// Source is Remote for Download
		src = Aspera::FaspManager::Location::CreateRemote( "demo.asperasoft.com", "aspera" , "demoaspera" );
		src->AddFilepath( "aspera-test-dir-small/smallfile00", "downloaded_small_test_file" );
		// Destination is Local for Download
	    dst = Aspera::FaspManager::Location::CreateLocal( );
	    // Remember to check the destination_root set in the xfer_params :D
	

	    /* Uncomment the following to perform an upload */
	    /* Setup the Local and Remote endpoints for an upload */
		// Source is Local for Upload
	    //src = Aspera::FaspManager::Location::CreateLocal( );
	    //src->AddFilepath( "C:/Temp/testfile", "Upload/" );
		// Destination is Remote for Upload
	    //dst = Aspera::FaspManager::Location::CreateRemote( "demo.asperasoft.com", "aspera" , "demoaspera" );
		// Remember to check the destination_root set in the xfer_params :D

	    /* Setup the transfer parameters */
	    xfer_params = new Aspera::FaspManager::XferParams( );
	    xfer_params->encryption = Aspera::FaspManager::ENCRYPT_AES128;
	    xfer_params->target_rate = TARGET_RATE; /* Transfers never exceed this rate  */
	    xfer_params->min_rate = MIN_RATE;
	    xfer_params->policy = Aspera::FaspManager::POLICY_FAIR;
	    xfer_params->create_dirs = CREATE_DIRS;
	    xfer_params->resume_check = Aspera::FaspManager::RESUME_NONE;
		xfer_params->destination_root = "";

	    /* Setup the job order */
	    job_defn = new Aspera::FaspManager::JobOrder( );
	    job_defn->src_location = src;
	    job_defn->dst_location = dst;
	    job_defn->xfer_params = xfer_params;

		/* Setup a callback function */
	    Aspera::FaspManager::CallbackFunctor<MyCallbackClass> *callback;
	    MyCallbackClass *callbackobj = new MyCallbackClass();
	    callback = new Aspera::FaspManager::CallbackFunctor<MyCallbackClass>( callbackobj, &MyCallbackClass::ReceiveCallback );

	    /* Get the FaspManager Instance */
		manager = Aspera::FaspManager::Manager::GetInstance();
	    manager->SetDebugLevel(2);
	    //manager->AddGlobalCallback(callback);
	    //manager->SetManagementPortPublic(true); // If using fasp manager on the server, uncomment this line

	    /* Launch the job */
		const char* job_id;
	    job_id = manager->LaunchJob ( job_defn, callback );

		/*  Wait for session to start  */
		while(!callbackobj->isSessionReady())
			callbackobj->sleep(1000);

		// Once your job had been passed to the manager, you can clear the memory.
		delete (job_defn);
	
		/* Let it run until finished or an error occurs  */
		while(callbackobj->isSessionReady())
			 callbackobj->sleep(1000);

	} catch (Aspera::FaspManager::Exception& e) {
		printf("Exception thrown: %s\n", e.GetErrMsg());
	} catch (...) {
		printf("Exception thrown! - We should never see this, the exception must be thrown incorrectly.\n");
	}
	
	printf("\nPress Enter to end the demo...\n"); 
	getc( stdin );

	manager->Destroy();
	 
	return 0;
}
